# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2023 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Location test case
"""
import os
from flask.json import dumps as json_dumps
from flask.json import loads as json_loads
import pytest
import unittest
from actinia_core.core.common.app import flask_app
from actinia_core.core.common.config import global_config
from actinia_core.core.common import redis_interface
from actinia_core.core.common.process_queue import create_process_queue
from actinia_core.testsuite import (
    ActiniaRequests,
    ActiniaTestCaseBase,
    URL_PREFIX,
)

__license__ = "GPLv3"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2023, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"
__email__ = "info@mundialis.de"


# Set this variable to use a actinia config file in a docker container
if "ACTINIA_CUSTOM_TEST_CFG" in os.environ:
    custom_actinia_cfg = str(os.environ["ACTINIA_CUSTOM_TEST_CFG"])

PC = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "elevation@PERMANENT"}],
            "flags": "p",
        },
        {
            "id": "2",
            "module": "r.colors.out_sld",
            "inputs": [
                {"param": "map", "value": "elevation@PERMANENT"},
                {"param": "style_name", "value": "elevation"},
            ],
            "stdout": {"id": "sld", "format": "list", "delimiter": "\n"},
        },
        {
            "id": "3",
            "module": "r.mapcalc",
            "inputs": [{"param": "expression", "value": "baum = 5"}],
        },
        {
            "id": "4",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "elevation@PERMANENT"}],
            "flags": "g",
            "stdout": {"id": "r_info_map1", "format": "kv", "delimiter": "="},
        },
        {
            "id": "5",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "6",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "baum"}],
            "flags": "g",
            "stdout": {"id": "r_info_map2", "format": "kv", "delimiter": "="},
        },
    ],
}


class ActiniaWithoutAuthentication(ActiniaTestCaseBase):
    """Test base class to test actinia without autentication"""

    server_test = False

    if "ACTINIA_SERVER_TEST" in os.environ:
        server_test = bool(os.environ["ACTINIA_SERVER_TEST"])

    @classmethod
    def setUpClass(cls):
        if cls.server_test is False and cls.custom_actinia_cfg is False:
            global_config.REDIS_SERVER_SERVER = "localhost"
            global_config.REDIS_SERVER_PORT = 7000
            global_config.GRASS_RESOURCE_DIR = "/tmp"
            global_config.DOWNLOAD_CACHE = "/tmp/download_cache"
            global_config.REDIS_QUEUE_SERVER_URL = "localhost"
            global_config.REDIS_QUEUE_SERVER_PORT = 6379
            global_config.NUMBER_OF_WORKERS = 3

        # If the custom_actinia_cfg variable is set, then the actinia config
        # file will be read to configure Redis queue
        if cls.server_test is False and cls.custom_actinia_cfg is not False:
            global_config.read(cls.custom_actinia_cfg)
            # global_config.read(cls.custom_actinia_cfg)
        # Start the redis interface
        redis_args = (
            global_config.REDIS_SERVER_URL,
            global_config.REDIS_SERVER_PORT,
        )
        if (
            global_config.REDIS_SERVER_PW
            and global_config.REDIS_SERVER_PW is not None
        ):
            redis_args = (*redis_args, global_config.REDIS_SERVER_PW)

        redis_interface.connect(*redis_args)

        # Process queue
        create_process_queue(config=global_config)

    @classmethod
    def tearDownClass(cls):
        if cls.server_test is False:
            redis_interface.disconnect()

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()

        # Check if the local or server site tests should be performed
        if self.server_test is False:
            flask_app.config["TESTING"] = True

            self.server = flask_app.test_client()
        else:
            self.server = ActiniaRequests()

    def tearDown(self):
        self.app_context.pop()

    def waitAsyncStatusAssertHTTP(
        self,
        response,
        http_status=200,
        status="finished",
        message_check=None,
    ):
        """Poll the status of a resource and assert its finished HTTP status

        The response will be checked if the resource was accepted. Hence it
        must always be HTTP 200 status.

        The status URL from the response is then polled until status: finished,
        error or terminated.
        The result of the poll can be checked against its HTTP status and its
        actinia status message.

        Args:
            response: The accept response
            http_status (int): The HTTP status that should be checked
            status (str): The return status of the response
            message_check (str): A string that must be in the message field

        Returns: response

        """
        resp_data = super().waitAsyncStatusAssertHTTP(
            response, None, http_status, status, message_check
        )
        return resp_data


@pytest.mark.noauth
class TestActiniaWithoutAuthentication(ActiniaWithoutAuthentication):
    def test_01_version(self):
        """Test version endpoint"""
        rv = self.server.get(f"{URL_PREFIX}/version")
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        data = json_loads(rv.data)
        self.assertIn("api_version", data)
        self.assertIn("version", data)
        self.assertIn("grass_version", data)

    def test_02_list_locations(self):
        """Test list location endpoint"""
        rv = self.server.get(f"{URL_PREFIX}/locations")
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        self.assertIn(
            "locations", json_loads(rv.data), "No locations in response"
        )
        locations = json_loads(rv.data)["locations"]
        self.assertIn("nc_spm_08", locations, "Wrong location listed")

    def test_03_processing_ephemeral(self):
        """Test job resumption with processing_async endpoint and stdout"""
        endpoint = "/locations/nc_spm_08/processing_async"
        rv = self.server.post(
            f"{URL_PREFIX}{endpoint}",
            data=json_dumps(PC),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            http_status=200,
            status="finished",
        )
        self.compare_stdout(resp)

    def test_04_processing_persistent(self):
        """Test job resumption with persistent processing_async endpoint and
        stdout
        """
        endpoint = "/locations/nc_spm_08/mapsets/test/processing_async"
        rv = self.server.post(
            f"{URL_PREFIX}{endpoint}",
            data=json_dumps(PC),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            http_status=200,
            status="finished",
        )
        self.compare_stdout(resp)
        # check processing mapset
        rv2 = self.server.get(f"{URL_PREFIX}/locations/nc_spm_08/mapsets")
        self.assertEqual(
            rv2.status_code,
            200,
            f"HTML status code is wrong {rv2.status_code}",
        )
        data = json_loads(rv2.data)
        self.assertIn(
            "test", data["process_results"], "Mapset 'test' does not exists"
        )
        # check created raster
        rv3 = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/test/raster_layers"
        )
        self.assertEqual(
            rv3.status_code,
            200,
            f"HTML status code is wrong {rv3.status_code}",
        )
        data = json_loads(rv3.data)
        self.assertIn(
            "baum", data["process_results"], "Raster 'baum' does not exists"
        )
        # delete test mapset
        self.admin_auth_header = None
        self.delete_mapset("test", "nc_spm_08")
        rv4 = self.server.get(f"{URL_PREFIX}/locations/nc_spm_08/mapsets")
        self.assertEqual(
            rv4.status_code,
            200,
            f"HTML status code is wrong {rv4.status_code}",
        )
        data = json_loads(rv4.data)
        self.assertNotIn(
            "test", data["process_results"], "Mapset 'test' exists"
        )

    def compare_stdout(self, resp):
        proc_results = resp["process_results"]
        self.assertIn(
            "sld", proc_results, "'sld' not saved in process results"
        )
        self.assertIn(
            "r_info_map1",
            proc_results,
            "'r_info_map1' not saved in process results",
        )
        self.assertIn(
            "r_info_map2",
            proc_results,
            "'r_info_map2' not saved in process results",
        )
        self.assertIsInstance(
            proc_results["sld"], list, "'sld' result is not of type list"
        )
        self.assertIsInstance(
            proc_results["r_info_map1"],
            dict,
            "'r_info_map1' result is not of type dict",
        )
        self.assertIsInstance(
            proc_results["r_info_map2"],
            dict,
            "'r_info_map2' result is not of type dict",
        )
        r_info_map1 = {
            "cells": "2025000",
            "cols": "1500",
            "datatype": "FCELL",
            "east": "645000",
            "ewres": "10",
            "ncats": "255",
            "north": "228500",
            "nsres": "10",
            "rows": "1350",
            "south": "215000",
            "west": "630000",
        }
        self.assertEqual(
            proc_results["r_info_map1"],
            r_info_map1,
            "'r_info_map1' is not equal to the region",
        )
        r_info_map2 = {
            "cells": "2025000",
            "cols": "1500",
            "datatype": "CELL",
            "east": "645000",
            "ewres": "10",
            "ncats": "0",
            "north": "228500",
            "nsres": "10",
            "rows": "1350",
            "south": "215000",
            "west": "630000",
        }
        self.assertEqual(
            proc_results["r_info_map2"],
            r_info_map2,
            "'r_info_map2' is not equal to the region",
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Raster Layer Renderer test case
"""
import unittest
from pprint import pprint
from flask.json import loads as json_load

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX


__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class RasterLayerRendererTestCase(ActiniaResourceTestCaseBase):
    def test_raster_layer_image_no_args(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/render",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/render?n=228500&s=215000&w=630000&e=645000",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/render?n=228500&s=215000&w=630000&e=645000&"
            "width=100&height=100",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_3(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/render?width=100&height=100",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_error_1(self):
        # North is smaller then south
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/render?n=-228500&s=215000",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_error_2(self):
        # Negative size
        rv = self.server.get(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/PERMANENT/raster_layers/elevation/"
            "render?&width=-100&height=-100",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_args_error_3(self):
        # Raster does not exist
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevat/render?",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "red=lsat5_1987_10&blue=lsat5_1987_20&green=lsat5_1987_30",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "n=228513&s=214975.5&w=629992.5&e=645012&width=100&height=100"
            "&red=lsat5_1987_30&blue=lsat5_1987_20&green=lsat5_1987_10",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_3(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "width=100&height=100"
            "&red=lsat5_1987_30&blue=lsat5_1987_20&green=lsat5_1987_10",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_green(self):
        # No green raster layer
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "n=228513&s=214975.5&w=629992.5&e=645012"
            "&red=lsat5_1987_30&blue=lsat5_1987_20",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_blue(self):
        # No blue raster layer
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "n=228513&s=214975.5&w=629992.5&e=645012"
            "&red=lsat5_1987_30&green=lsat5_1987_20",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_red(self):
        # No red raster layer
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "n=228513&s=214975.5&w=629992.5&e=645012"
            "&blue=lsat5_1987_30&green=lsat5_1987_20",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_wrong_raster(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "n=228513&s=214975.5&w=629992.5&e=645012"
            "&red=lsat5_1987_30_1&blue=lsat5_1987_20&green=lsat5_1987_10",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_mapset_in_name_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "red=lsat5_1987_10@landsat&blue=lsat5_1987_20&green=lsat5_1987_30",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_mapset_in_name_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "red=lsat5_1987_10&blue=lsat5_1987_20@landsat&green=lsat5_1987_30",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_rgb_error_mapset_in_name_3(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/landsat/render_rgb?"
            "red=lsat5_1987_10&blue=lsat5_1987_20&green=lsat5_1987_30@landsat",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_shade_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/render_shade?"
            "shade=aspect&color=elevation",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_shade_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/render_shade?"
            "width=100&height=100"
            "&shade=aspect&color=elevation",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_error_mapset_in_name_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/render_shade?"
            "&shade=aspect@PERMANENT&color=elevation",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_error_mapset_in_name_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/render_shade?"
            "&shade=aspect&color=elevation@PERMANENT",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_image_error_missing_color_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/render_shade?"
            "&shade=aspect",
            headers=self.user_auth_header,
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Asynchronous computation in specific temporary generated mapsets
with export of required map layers.
"""
import pickle
from flask import jsonify, make_response

from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import (
    ephemeral_processing_with_export,
)

from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.ephemeral_processing_with_export import (
    start_job,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"
__email__ = "info@mundialis.de"


class AsyncEphemeralExportResource(ResourceBase):
    """
    This class represents a resource that runs asynchronous processing tasks in
    a temporary mapset and exports the computed results as geotiff files.
    """

    def __init__(self, resource_id=None, iteration=None, post_url=None):
        ResourceBase.__init__(self, resource_id, iteration, post_url)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("post", ephemeral_processing_with_export.post_doc)
    )
    def post(self, location_name):
        """Execute a user defined process chain in an ephemeral location/mapset
        and store the processing results for download.
        """
        rdc = self.preprocess(has_json=True, location_name=location_name)

        if rdc:
            rdc.set_storage_model_to_file()
            enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)


class AsyncEphemeralExportS3Resource(ResourceBase):
    """
    This class represents a resource that runs asynchronous processing tasks in
    a temporary mapset and exports the computed results as geotiff files to the
    Amazon S3 storage.
    """

    def __init__(self):
        ResourceBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("post", ephemeral_processing_with_export.post_doc)
    )
    def post(self, location_name):
        """Execute a user defined process chain in an ephemeral location/mapset
        and store the processing result in an Amazon S3 bucket
        """
        rdc = self.preprocess(has_json=True, location_name=location_name)
        rdc.set_storage_model_to_s3()

        enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)


class AsyncEphemeralExportGCSResource(ResourceBase):
    """
    This class represents a resource that runs asynchronous processing tasks in
    a temporary mapset and exports the computed results as geotiff files to the
    Google Cloud Storage.
    """

    def __init__(self):
        ResourceBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("post", ephemeral_processing_with_export.post_doc)
    )
    def post(self, location_name):
        """Execute a user defined process chain in an ephemeral location/mapset
        and store the processing result in an Google cloud storage bucket
        """
        rdc = self.preprocess(has_json=True, location_name=location_name)
        rdc.set_storage_model_to_gcs()

        enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Resource logging test case
"""
import unittest
import pickle
import uuid
from actinia_core.core.resources_logger import ResourceLogger
from actinia_core.core.common.app import flask_app

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, global_config
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, global_config

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class ResourceLoggingTestCase(ActiniaResourceTestCaseBase):
    """
    This class tests the resource logging interface
    """

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()
        # The test user
        self.user_id = "soeren"
        self.resource_id = uuid.uuid1()
        self.document = pickle.dumps(
            [200, {"Status": "running", "URL": "/bla/bla"}]
        )

        redis_args = (
            global_config.REDIS_SERVER_URL,
            global_config.REDIS_SERVER_PORT,
        )
        if global_config.REDIS_SERVER_PW is not None:
            redis_args = (*redis_args, global_config.REDIS_SERVER_PW)
        self.log = ResourceLogger(*redis_args)
        del redis_args

    def tearDown(self):
        self.app_context.pop()

    def test_logging(self):
        ret = self.log.commit(
            user_id=self.user_id,
            resource_id=self.resource_id,
            iteration=1,
            document=self.document,
        )

        self.assertTrue(ret)

        ret = self.log.commit(
            user_id=self.user_id,
            resource_id=self.resource_id,
            iteration=1,
            document=self.document,
        )

        self.assertTrue(ret)

        doc = self.log.get(
            user_id=self.user_id, resource_id=self.resource_id, iteration=1
        )
        print(doc)
        self.assertEqual(self.document, doc)

        ret = self.log.delete(
            user_id=self.user_id, resource_id=self.resource_id, iteration=1
        )

        self.assertTrue(ret)

        doc = self.log.get(user_id=self.user_id, resource_id=self.resource_id)

        self.assertEqual(None, doc)

        ret = self.log.delete(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertFalse(ret)

    def test_list(self):
        user = "lisa"
        resource = "a"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        user = "franky"
        resource = "a"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        resource = "b"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        user = "klaus"
        resource = "a"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        resource = "b"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        resource = "c"
        ret = self.log.commit(
            user_id=user,
            resource_id=resource,
            iteration=1,
            document=self.document,
        )
        self.assertTrue(ret)

        ret = self.log.get_user_resources("lisa")
        self.assertEqual(len(ret), 1)
        print(ret)

        ret = self.log.get_user_resources("franky")
        print(ret)

        ret = self.log.get_user_resources("klaus")
        print(ret)

    def test_termination(self):
        ret = self.log.commit_termination(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertTrue(ret)

        ret = self.log.get_termination(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertEqual(True, ret)

        ret = self.log.delete_termination(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertTrue(ret)

        ret = self.log.get_termination(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertEqual(False, ret)

        ret = self.log.delete_termination(
            user_id=self.user_id, resource_id=self.resource_id
        )

        self.assertFalse(ret)


if __name__ == "__main__":
    unittest.main()



#!flask/bin/python
# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Main file of the Actinia Core API
"""

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

from flask import make_response
from actinia_api import URL_PREFIX

from actinia_core.core.common.app import flask_app

# This is a simple endpoint to check the health of the Actinia Core server
# This is needed by Google load balancer


@flask_app.route(URL_PREFIX + "/health_check")
def health_check():
    # TODO: Add checks of the Actinia Core compute node state
    #       - is the config correct
    #       - Are the queues running
    #       - Is redis available
    # TODO: This check must be triggered to respond with 404 for node
    #       replacement in case of an update or bugfix.
    #       Hence, the load balance will not deliver any content to this node
    #       if the health check responses with a 404.
    return make_response("OK", 200)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
This module is designed to deliver the API calls of a specific user
"""
from flask import g
from flask import jsonify, make_response
from flask_restful import Resource
from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import api_log_management
from actinia_api.swagger2.actinia_core.schemas.api_log_management import (
    ApiLogListModel,
)

from actinia_core.core.common.app import auth
from actinia_core.core.common.api_logger import ApiLogger
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.models.response_models import SimpleResponseModel
from actinia_core.rest.base.user_auth import check_user_permissions

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class APILogResource(Resource):
    """API logg management"""

    decorators = [log_api_call, check_user_permissions, auth.login_required]

    def __init__(self):
        # Configuration
        Resource.__init__(self)

        self.api_logger = ApiLogger()

        # Store the user id and user role of the current user
        self.user_id = g.user.get_id()
        self.user_role = g.user.get_role()

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", api_log_management.get_doc))
    def get(self, user_id):
        """
        Get a list of all API calls that have been called by the provided user.
        """

        if (
            self.user_role not in ["admin", "superadmin"]
            and user_id != self.user_id
        ):
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="You do not have the permission "
                        "to list the API calls of the user",
                    )
                ),
                401,
            )

        api_log_list = self.api_logger.list(user_id, 0, -1)
        return make_response(
            jsonify(ApiLogListModel(api_log_list=api_log_list)), 200
        )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: AWS Sentinel interface test case
"""
from actinia_core.core.common.aws_sentinel_interface import (
    AWSSentinel2AInterface,
)
from actinia_core.core.common.config import global_config
import unittest
from pprint import pprint
import magic
import os
from urllib.request import urlopen

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class AWSSentinelInterfaceTestCase(unittest.TestCase):
    """This class tests theAWS interface to collect download
    urls of sentinel data
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def validate_result(self, result, bands):
        for scene in result:
            for tile in scene["tiles"]:
                url = tile["info"]
                response = urlopen(url)
                mime_type = magic.from_buffer(
                    response.read(256), mime=True
                ).lower()
                print(mime_type)
                self.assertTrue(mime_type in ["text/plain"])

                url = tile["metadata"]
                response = urlopen(url)
                mime_type = magic.from_buffer(
                    response.read(256), mime=True
                ).lower()
                print(mime_type)
                self.assertTrue(
                    mime_type in ["application/xml"]
                    or mime_type in ["text/xml"]
                )

                url = tile["preview"]
                response = urlopen(url)
                mime_type = magic.from_buffer(
                    response.read(256), mime=True
                ).lower()
                print(mime_type)
                self.assertTrue(mime_type in ["image/jpeg"])

                for band in bands:
                    url = tile[band]["public_url"]

                    # Download 256 bytes from the url and check its mimetype
                    response = urlopen(url)
                    mime_type = magic.from_buffer(
                        response.read(256), mime=True
                    ).lower()
                    print(mime_type)

                    self.assertTrue(mime_type in ["image/jp2"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_single_old(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_"
                "V20151207T003302_20151207T003302",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207T003302_"
            "20151207T003302" in result[0]["product_id"]
        )
        self.validate_result(result, ["B12", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_single_old_geojson(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207"
                "T003302_20151207T003302",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207"
            "T003302_20151207T003302" in result[0]["product_id"]
        )

        for tile in result[0]["tiles"]:
            geojson = aws.get_sentinel_tile_footprint(tile)
            print(geojson)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_single_new(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138"
            in result[0]["product_id"]
        )
        self.validate_result(result, ["B12", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_single_new_safe(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212"
                "T104138.SAFE",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138"
            in result[0]["product_id"]
        )
        self.validate_result(result, ["B12", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_singler_eplacement_bug(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_MSIL1C_20171210T020751_N0206_R017_T50HPG_20171210T052001",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_MSIL1C_20171210T020751_N0206_R017_T50HPG_20171210T052001"
            in result[0]["product_id"]
        )
        self.validate_result(result, ["B12", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_singler_eplacement_bug_2(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_OPER_PRD_MSIL1C_PDMC_20161031T014722_R087_V20161030"
                "T235752_20161030T235752",
            ],
            ["B12", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_OPER_PRD_MSIL1C_PDMC_20161031T014722_R087_V20161030"
            "T235752_20161030T235752" in result[0]["product_id"]
        )
        self.validate_result(result, ["B12", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_mutli_mixed(self):
        aws = AWSSentinel2AInterface(global_config)
        result = aws.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_20170202T090155",
                "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207"
                "T003302_20151207T003302",
                "S2A_MSIL1C_20170218T143751_N0204_R096_T20PRT_20170218T143931",
            ],
            ["B04", "B08"],
        )
        pprint(result)

        self.assertTrue(
            "S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_20170202T090155"
            in result[0]["product_id"]
        )
        self.assertTrue(
            "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207T003302_"
            "20151207T003302" in result[1]["product_id"]
        )
        self.assertTrue(
            "S2A_MSIL1C_20170218T143751_N0204_R096_T20PRT_20170218T143931"
            in result[2]["product_id"]
        )
        self.validate_result(result, ["B04", "B08"])

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_single_error(self):
        aws = AWSSentinel2AInterface(global_config)

        try:
            result = aws.get_sentinel_urls(
                [
                    "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212"
                    "T104138_NOPE",
                ],
                ["B12", "B08"],
            )
            pprint(result)
            self.assertTrue(False, "The error was not found")
        except Exception as e:
            self.assertTrue(
                True,
                "An exception was raised for the correct reason: %s" % str(e),
            )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_multi_error(self):
        aws = AWSSentinel2AInterface(global_config)

        try:
            result = aws.get_sentinel_urls(
                [
                    "S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_"
                    "20170202T090155",
                    "S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_"
                    "V20151207T003302_20151207T003302",
                    "S2A_MSIL1C_20170218T143751_N0204_R096_T20PRT_"
                    "20170218T143931_NOPE",
                ],
                ["B04", "B08"],
            )
            pprint(result)
            self.assertTrue(False, "The error was not found")
        except Exception as e:
            self.assertTrue(
                True,
                "An exception was raised for the correct reason: %s" % str(e),
            )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def otest_query_for_sentinel_scenes_band_error(self):
        aws = AWSSentinel2AInterface(global_config)

        try:
            aws.get_sentinel_urls(
                [
                    "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_"
                    "20170212T104138_NOPE",
                ],
                ["B120", "B080"],
            )
            # pprint(result)
            self.assertTrue(False, "The error was not found")
        except Exception as e:
            self.assertTrue(
                True,
                "An exception was raised for the correct reason: %s" % str(e),
            )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster map renderer
"""

import os
from tempfile import NamedTemporaryFile
from actinia_core.processing.actinia_processing.ephemeral_processing import (
    EphemeralProcessing,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class EphemeralRasterLegend(EphemeralProcessing):
    def __init__(self, *args):
        EphemeralProcessing.__init__(self, *args)

    def _execute(self, skip_permission_check=True):
        """Render the raster legend with d.legend"""
        self._setup()

        self.required_mapsets.append(self.mapset_name)
        raster_name = self.map_name

        options = self.rdc.user_data

        with NamedTemporaryFile(suffix=".png") as file:
            result_file = file.name

        os.putenv("GRASS_RENDER_IMMEDIATE", "png")
        os.putenv("GRASS_RENDER_WIDTH", str(options["width"]))
        os.putenv("GRASS_RENDER_HEIGHT", str(options["height"]))
        os.putenv("GRASS_RENDER_TRANSPARENT", "TRUE")
        os.putenv("GRASS_RENDER_TRUECOLOR", "TRUE")
        os.putenv("GRASS_RENDER_FILE", result_file)
        os.putenv("GRASS_RENDER_FILE_READ", "TRUE")

        pc = {}
        pc["1"] = {
            "module": "d.legend",
            "inputs": {"raster": raster_name + "@" + self.mapset_name},
        }
        for key in options:
            if key not in ["width", "height"]:
                value = options[key]
                pc["1"]["inputs"][key] = value

        process_list = (
            self._create_temporary_grass_environment_and_process_list(
                process_chain=pc, skip_permission_check=True
            )
        )
        self._execute_process_list(process_list)

        self.module_results = result_file



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Asynchronous merging of several mapsets into a single one
"""
from actinia_core.processing.actinia_processing.ephemeral.persistent_processing import (
    PersistentProcessing,
)
from actinia_core.core.common.exceptions import (
    AsyncProcessError,
    AsyncProcessTermination,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class PersistentMapsetMerger(PersistentProcessing):
    """Processing of grass modules in a temporary or original mapset.

    This class is designed to run GRASS modules that are specified in a process
    chain in a temporary mapset that later on is copied into the original
    location.

    If the processing should be run in an existing mapset, the original mapset
    will be used for processing.
    """

    def __init__(self, rdc):
        """Constructor

        Args:
            rdc (ResourceDataContainer): The data container that contains all
                                         required variables for processing

        """

        PersistentProcessing.__init__(self, rdc)
        self.lock_ids = (
            {}
        )  # This dict holds the lock ids of all locked mapsets

    def _check_lock_mapset(self, mapset_name):
        """Check if the mapset exists and lock it

        If the mapset is a global mapset and Error will be raised.

        The duration of the lock is process_time_limit * process_num_limit
        and should be extended if needed.

        Only mapsets of the user database are locked.

        Unlock the mapset after the processing finished.
        """
        # check if the resource is accessible
        mapset_exists = self._check_mapset(mapset_name)

        if mapset_exists is False:
            raise AsyncProcessError(
                "Mapset <%s> does not exist and can not be locked."
                % mapset_name
            )

        # Finally lock the mapset for the time that the user can allocate at
        # maximum
        lock_id = "%s/%s/%s" % (
            self.user_group,
            self.location_name,
            mapset_name,
        )
        ret = self.lock_interface.lock(
            resource_id=lock_id,
            expiration=self.process_time_limit * self.process_num_limit,
        )

        if ret == 0:
            raise AsyncProcessError(
                "Unable to lock mapset <%s>, resource is already locked"
                % mapset_name
            )
        self.message_logger.info("Mapset <%s> locked" % mapset_name)

        # if we manage to come here, the lock was correctly set, hence store
        # the lock id for later unlocking
        self.lock_ids[lock_id] = mapset_name

    def _check_lock_source_mapsets(self, source_mapsets):
        """Check and lock the source mapsets from the merging list

        Args:
            source_mapsets: A list of source mapsets that should be checked
                            and locked

        Raises:
            This method will raise an AsyncProcessError

        """
        # Expect a list of mapset names
        if len(source_mapsets) == 0:
            raise AsyncProcessError("Empty source mapset list.")

        # Check and lock the mapsets
        for mapset in source_mapsets:
            self._check_lock_mapset(mapset)

    def _merge_mapsets(self):
        """Merge mapsets in a target mapset

        - Check the target mapset and lock it for the maximum time
          a user can consume -> process_num_limit*process_time_limit
        - Check and lock all source mapsets with the same scheme
        - Copy each source mapset into the target mapset
        - Extend the locks each copy run
        - Cleanup and unlock the mapsets

        """
        # Lock the target mapset
        self._check_lock_mapset(self.target_mapset_name)
        # Lock the source mapsets
        self._check_lock_source_mapsets(self.request_data)

        step = 1
        steps = len(self.request_data)

        mapsets_to_merge = []

        # Copy each mapset into the target
        for lock_id in self.lock_ids:
            # Check for termination requests
            if (
                self.resource_logger.get_termination(
                    self.user_id, self.resource_id
                )
                is True
            ):
                raise AsyncProcessTermination(
                    "Mapset merging was terminated "
                    "by user request at setp %i of %i" % (step, steps)
                )

            mapset_name = self.lock_ids[lock_id]
            mapsets_to_merge.append(mapset_name)

            for lock_id in self.lock_ids:
                # Extent the lock for each process by max processing time * 2
                ret = self.lock_interface.extend(
                    resource_id=lock_id, expiration=self.process_time_limit * 2
                )
                if ret == 0:
                    raise AsyncProcessError(
                        "Unable to extend lock for mapset <%s>" % mapset_name
                    )

            message = (
                "Step %i of %i: Copy content from source "
                "mapset <%s> into target mapset <%s>"
                % (step, steps, mapset_name, self.target_mapset_name)
            )
            self._send_resource_update(message)
            self.message_logger.info(message)

            # Copy the source mapset into the target mapset
            if mapset_name != self.target_mapset_name:
                step += 1
                self._merge_mapset_into_target(
                    mapset_name, self.target_mapset_name
                )

    def _execute(self):
        """The _execute() function that does all the magic.

        Overwrite this function in subclasses.

            - Check the target mapset and lock it for the maximum time
              a user can consume -> process_num_limit*process_time_limit
            - Check and lock all source mapsets with the same scheme
            - Copy each source mapset into the target mapset
            - Extend the locks each copy run
            - Cleanup and unlock the mapsets

        """
        # Setup the user credentials and logger
        self._setup()
        self._merge_mapsets()

    def _final_cleanup(self):
        """
        Final cleanup called in the run function at the very end of processing
        """
        # Clean up and remove the temporary gisdbase
        # Unlock mapsets
        PersistentProcessing._final_cleanup(self)
        # Unlock the mapsets
        for lock_id in self.lock_ids:
            self.lock_interface.unlock(lock_id)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster map renderer
"""

import os
from flask import jsonify, make_response, Response
from flask_restful import reqparse
from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import raster_legend

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.raster_legend import start_job


__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class SyncEphemeralRasterLegendResource(ResourceBase):
    """Render the raster legend with d.legend"""

    def create_parser(self):
        # Create a legend parser
        parser = reqparse.RequestParser()
        parser.add_argument(
            "at",
            type=str,
            location="args",
            help="Size and placement as percentage of "
            "screen coordinates (0,0 is lower left). "
            "bottom,top,left,right 0-100%",
        )
        parser.add_argument(
            "range",
            type=str,
            location="args",
            help="Use a subset of the map range for the legend " "(min,max)",
        )
        parser.add_argument(
            "use",
            type=str,
            location="args",
            help="List of discrete category numbers/values for legend",
        )
        parser.add_argument(
            "fontsize",
            type=float,
            location="args",
            help="Size of the font 1-360",
        )
        parser.add_argument(
            "labelnum",
            type=float,
            location="args",
            help="Number of text labels for smooth gradient legend",
        )
        parser.add_argument(
            "width",
            type=float,
            location="args",
            help="North-South resolution must be specified as " "double value",
        )
        parser.add_argument(
            "height",
            type=float,
            location="args",
            help="East-West resolution must be specified as " "double value",
        )

        return parser

    def create_parser_options(self, args):
        """Parse the request parameters for raster legends

        Args:
            args (dict): The argument dictionary with display options

        Returns:
             dict:
             A dictionary of display parameter or an error response in case of
             an error
        """
        options = {}
        options["width"] = 600
        options["height"] = 800
        options["at"] = "0,100,0,15"

        if "labelnum" in args and args["labelnum"] is not None:
            options["labelnum"] = args["labelnum"]
        if "use" in args and args["use"] is not None:
            options["use"] = args["use"]
        if "at" in args and args["at"] is not None:
            options["at"] = args["at"]
        if "range" in args and args["range"] is not None:
            options["range"] = args["range"]
        if "use" in args and args["use"] is not None:
            options["use"] = args["use"]
        if "fontsize" in args and args["fontsize"] is not None:
            options["fontsize"] = args["fontsize"]
        if "width" in args and args["width"] is not None:
            if args["width"] < 1:
                return self.get_error_response(
                    message="Width must be larger than 0"
                )
            if args["width"] > 10000:
                return self.get_error_response(
                    message="Width can not be larger than 10000"
                )
            options["width"] = args["width"]
        if "height" in args and args["height"] is not None:
            if args["height"] < 1:
                return self.get_error_response(
                    message="Height must be larger than 0"
                )
            if args["height"] > 10000:
                return self.get_error_response(
                    message="Height can not be larger than 10000"
                )
            options["height"] = args["height"]

        return options

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", raster_legend.get_doc))
    def get(self, location_name, mapset_name, raster_name):
        """Render the legend of a raster map layer as a PNG image."""
        parser = self.create_parser()
        args = parser.parse_args()
        options = self.create_parser_options(args)

        if isinstance(options, dict) is False:
            return options

        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=raster_name,
        )

        rdc.set_user_data(options)

        enqueue_job(
            self.job_timeout, start_job, rdc, queue_type_overwrite=True
        )

        http_code, response_model = self.wait_until_finish(0.05)
        if http_code == 200:
            result_file = response_model["process_results"]
            # Open the image file, read it and then delete it
            if result_file:
                if os.path.isfile(result_file):
                    image = open(result_file, "rb").read()
                    os.remove(result_file)
                    return Response(image, mimetype="image/png")
        return make_response(jsonify(response_model), http_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Base class for user management resources
"""

from flask_restful import Resource
from actinia_core.core.common.app import auth
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.core.messages_logger import MessageLogger
from actinia_core.rest.base.user_auth import check_admin_role

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class LoginBase(Resource):
    """
    Base class for all login resources related classes
    """

    # Authorization is required for all resources
    # API logging is required for all resources
    decorators = [log_api_call, check_admin_role, auth.login_required]

    def __init__(self):
        Resource.__init__(self)
        self.message_logger = MessageLogger()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Mapset resources for information across all locations

* List all mapset locks
* List all mapsets in all locations available to a user
"""

from flask import jsonify, make_response
from flask_restful_swagger_2 import swagger
from flask import request
from actinia_api.swagger2.actinia_core.apidocs import mapsets

from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.rest.base.user_auth import check_user_permissions

# from actinia_core.rest.base.user_auth import check_admin_role
from actinia_core.core.common.app import auth
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.core.common.config import global_config
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.redis_lock import RedisLockingInterface
from actinia_core.core.redis_user import RedisUserInterface
from actinia_core.models.response_models import (
    SimpleResponseModel,
    MapsetListResponseModel,
    LockedMapsetListResponseModel,
)


__license__ = "GPLv3"
__author__ = "Julia Haas, Guido Riembauer"
__copyright__ = "Copyright 2021 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class AllMapsetsListingResourceAdmin(ResourceBase):
    """Get all locked mapsets"""

    decorators = [log_api_call, check_user_permissions, auth.login_required]

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", mapsets.get_doc))
    def get(self):
        if "status" in request.args:
            if request.args["status"] == "locked":
                if self.user.has_superadmin_role() is False:
                    return make_response(
                        jsonify(
                            SimpleResponseModel(
                                status="error",
                                message=(
                                    "Unable to list locked mapsets You are not"
                                    " authorized for this request. "
                                    "Minimum required user role: superadmin"
                                ),
                            )
                        ),
                        401,
                    )
                redis_interface = RedisLockingInterface()
                kwargs = dict()
                kwargs["host"] = global_config.REDIS_SERVER_URL
                kwargs["port"] = global_config.REDIS_SERVER_PORT
                if (
                    global_config.REDIS_SERVER_PW
                    and global_config.REDIS_SERVER_PW is not None
                ):
                    kwargs["password"] = global_config.REDIS_SERVER_PW

                redis_interface.connect(**kwargs)
                redis_connection = redis_interface.redis_server
                keys_locked = redis_connection.keys("RESOURCE-LOCK*")
                redis_interface.disconnect()
                keys_locked_dec = [key.decode() for key in keys_locked]
                mapsets_locked = [
                    "/".join(key.split("/")[-2:]) for key in keys_locked_dec
                ]
                try:
                    return make_response(
                        jsonify(
                            LockedMapsetListResponseModel(
                                status="success",
                                message="number of locked mapsets: %s"
                                % len(mapsets_locked),
                                locked_mapsets_list=mapsets_locked,
                            )
                        ),
                        200,
                    )

                except Exception as e:
                    return make_response(
                        jsonify(
                            SimpleResponseModel(
                                status="error",
                                message="Unable to list locked mapsets: "
                                f"Exception {e}",
                            )
                        ),
                        500,
                    )
        else:
            redis_interface = RedisUserInterface()
            kwargs = dict()
            kwargs["host"] = global_config.REDIS_SERVER_URL
            kwargs["port"] = global_config.REDIS_SERVER_PORT
            if (
                global_config.REDIS_SERVER_PW
                and global_config.REDIS_SERVER_PW is not None
            ):
                kwargs["password"] = global_config.REDIS_SERVER_PW
            redis_interface.connect(**kwargs)
            if "user" in request.args:
                user = request.args["user"]
                if self.user.has_superadmin_role() is False:
                    redis_interface.disconnect()
                    return make_response(
                        jsonify(
                            SimpleResponseModel(
                                status="error",
                                message=(
                                    f"Unable to list mapsets for user {user}: "
                                    "You are not authorized for this request. "
                                    "Minimum required user role: superadmin"
                                ),
                            )
                        ),
                        401,
                    )
            else:
                user = self.user.get_id()
            locs_mapsets = redis_interface.get_credentials(user)[
                "permissions"
            ]["accessible_datasets"]
            redis_interface.disconnect()
            mapsets = []
            for location in locs_mapsets:
                for mapset in locs_mapsets[location]:
                    mapsets.append(f"{location}/{mapset}")
            try:
                return make_response(
                    jsonify(
                        MapsetListResponseModel(
                            status="success",
                            available_mapsets=mapsets,
                        )
                    ),
                    200,
                )

            except Exception as e:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message=f"Unable to list mapsets: Exception {e}",
                        )
                    ),
                    500,
                )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: User requests test case
"""
from pprint import pprint
from flask.json import dumps as json_dumps
from werkzeug.datastructures import Headers
from flask.json import loads as json_loads
import unittest
import time
import base64
from random import randint

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
from actinia_core.core.common.user import ActiniaUser

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class UserRequestsTestCase(ActiniaResourceTestCaseBase):
    def test_user_status_requests_1(self):
        """Resource list with 3 finished resources"""

        # Create a random test user id that are used for login as admin
        user_id = "heinz" + str(randint(0, 10000000))
        user_group = self.user_group
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="user",
            accessible_datasets={
                "nc_spm_08": [
                    "PERMANENT",
                    "user1",
                    "landsat",
                    "modis_lst",
                    "test_mapset",
                ],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=2,
        )

        # Create three successfully run resources
        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        # print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]

        pprint(resource_list)

        self.assertTrue(len(resource_list) == 3)

        # Count return stats
        finished = 0
        for resource in resource_list:
            finished += int(resource["status"] == "finished")
            # print(resource["status"])

        self.assertTrue(finished == 3)

        # Check the different resource list parameters
        rv = self.server.get(
            URL_PREFIX + "/resources/%s?num=1" % user_id, headers=auth_header
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 1)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?num=2" % user_id, headers=auth_header
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 2)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?num=2&type=finished" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 2)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?num=2&type=all" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 2)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?type=all" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 3)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?type=finished" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 3)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?type=terminated" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 0)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s?type=unknown" % user_id,
            headers=auth_header,
        )
        self.assertTrue(len(json_loads(rv.data)["resource_list"]) == 0)

        # Check permission access using the default users

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        # print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        # print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        # print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.root_auth_header,
        )
        # print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

    def test_user_status_requests_2(self):
        """Resource list with 2 finished, 1 terminated and 2 error resources"""

        # Create a random test user id that are used for login as admin
        user_id = "heinz" + str(randint(0, 10000000))
        user_group = self.user_group
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="admin",
            accessible_datasets={
                "nc_spm_08": ["PERMANENT", "user1", "landsat", "test_mapset"],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=5,
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=auth_header)

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=auth_header)

        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["10"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv, headers=auth_header, http_status=400, status="terminated"
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname_Fail",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv, headers=auth_header, http_status=400, status="error"
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/uname_Fail",
            headers=auth_header,
            data=json_dumps(["-a"]),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv, headers=auth_header, http_status=400, status="error"
        )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 5)

        # Count return stats
        error = 0
        finished = 0
        terminated = 0
        for resource in resource_list:
            print(resource["status"])
            finished += int(resource["status"] == "finished")
            error += int(resource["status"] == "error")
            terminated += int(resource["status"] == "terminated")

        self.assertTrue(finished == 2)
        self.assertTrue(terminated == 1)
        self.assertTrue(error == 2)

        # Check permission access using the default users

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.root_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_user_status_requests_3(self):
        """Empty resource list test"""

        # Create a random test user id that are used for login as admin
        user_id = "heinz" + str(randint(0, 10000000))
        user_group = "test"
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(user_id, user_group, password)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 0)

        # Check permission access using the default users

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.root_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_time_termination(self):
        """Test the time dependent termination of three running resources"""

        # Create a random test user id that are used for login as admin
        user_id = "heinz" + str(randint(0, 10000000))
        user_group = "test"
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="user",
            accessible_datasets={
                "nc_spm_08": ["PERMANENT", "user1", "landsat", "test_mapset"],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=2,
        )

        # Start three processes that exceeds the time limit
        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wait for termination
        time.sleep(5)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 3)

        # Count return stats
        terminated = 0
        for resource in resource_list:
            terminated += int(resource["status"] == "terminated")
            print(resource["status"])

        self.assertTrue(terminated == 3)

    def test_user_termination(self):
        """Test the termination of three running resources"""

        # Create a random test user id that are used for login as admin
        user_id = "heinz" + str(randint(0, 10000000))
        user_group = "test"
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="user",
            accessible_datasets={
                "nc_spm_08": ["PERMANENT", "user1", "landsat", "test_mapset"],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=100,
        )

        # Start three processes that will be terminated
        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        # Test guest termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test user termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test admin termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Delete the resources
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wait for termination
        time.sleep(5)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 3)

        # Count return stats
        terminated = 0
        for resource in resource_list:
            terminated += int(resource["status"] == "terminated")
            print(resource["status"])

        self.assertTrue(terminated == 3)

    def test_admin_termination(self):
        """Test the termination of one running resources from admin user"""

        # Create a random test user id that are used for login as admin
        user_id = "horst" + str(randint(0, 10000000))
        user_group = self.user_group
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="admin",
            accessible_datasets={
                "nc_spm_08": ["PERMANENT", "user1", "landsat", "test_mapset"],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=100,
        )

        # Start three processes that will be terminated
        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        # Test guest termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test user termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test admin termination success
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Test superadmin termination success
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.root_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wait for termination
        time.sleep(5)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 1)

        # Count return stats
        terminated = 0
        for resource in resource_list:
            terminated += int(resource["status"] == "terminated")
            print(resource["status"])

        self.assertTrue(terminated == 1)

    def test_superadmin_termination(self):
        """
        Test the termination of one running resources from superadmin user
        """

        # Create a random test user id that are used for login as admin
        user_id = "horst" + str(randint(0, 10000000))
        user_group = self.user_group
        password = "1234"

        # We need to create an HTML basic authorization header
        auth_header = Headers()
        auth = bytes("%s:%s" % (user_id, password), "utf-8")
        auth_header.add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(user_id)
        if user.exists():
            user.delete()
        # Create a user in the database and reduce its credentials
        self.user = ActiniaUser.create_user(
            user_id,
            user_group,
            password,
            user_role="superadmin",
            accessible_datasets={
                "nc_spm_08": ["PERMANENT", "user1", "landsat", "test_mapset"],
                "ECAD": ["PERMANENT"],
            },
            process_num_limit=3,
            process_time_limit=100,
        )

        # Start three processes that will be terminated
        rv = self.server.post(
            URL_PREFIX + "/custom_process/sleep",
            headers=auth_header,
            data=json_dumps(["20"]),
            content_type="application/json",
        )

        # Test guest termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.guest_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test user termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test admin termination error
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.admin_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            401,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # Test superadmin termination success
        rv = self.server.delete(
            URL_PREFIX + "/resources/%s" % user_id,
            headers=self.root_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wait for termination
        time.sleep(5)

        rv = self.server.get(
            URL_PREFIX + "/resources/%s" % user_id, headers=auth_header
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resource_list = json_loads(rv.data)["resource_list"]
        self.assertTrue(len(resource_list) == 1)

        # Count return stats
        terminated = 0
        for resource in resource_list:
            terminated += int(resource["status"] == "terminated")
            print(resource["status"])

        self.assertTrue(terminated == 1)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Vector layer renderer test case
"""
import unittest
from pprint import pprint
from flask.json import loads as json_load

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX


__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class VectorLayerRendererTestCase(ActiniaResourceTestCaseBase):
    def test_vectorlayer_image_no_args(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_1(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render?n=228500&s=215000&w=630000&e=645000",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_2(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render?n=228500&s=215000&w=630000&e=645000&"
            "width=100&height=100",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_3(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render?width=100&height=100",
            headers=self.user_auth_header,
        )

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "image/png", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_error_1(self):
        # North is smaller then south
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render?n=-228500&s=215000",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_error_2(self):
        # Negative size
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county/render?&width=-100&height=-100",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_vectorlayer_image_args_error_3(self):
        # Raster does not exist
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/vector_layers"
            "/boundary_county_nomap/render?",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Vector layer resources
"""
from flask import jsonify, make_response, request
from flask_restful_swagger_2 import swagger
import os
import pickle
from uuid import uuid4
from werkzeug.utils import secure_filename
from zipfile import ZipFile
from actinia_api.swagger2.actinia_core.apidocs import vector_layer

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.core.utils import allowed_file
from actinia_core.models.response_models import SimpleResponseModel
from actinia_core.rest.base.map_layer_base import MapLayerRegionResourceBase
from actinia_core.processing.common.vector_layer import (
    start_create_job,
    start_delete_job,
    start_info_job,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Guido Riembauer, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class VectorLayerResource(MapLayerRegionResourceBase):
    """Manage a vector map layer"""

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", vector_layer.get_doc))
    def get(self, location_name, mapset_name, vector_name):
        """Get information about an existing vector map layer."""
        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=vector_name,
        )

        if rdc:
            enqueue_job(
                self.job_timeout,
                start_info_job,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish(0.02)
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("delete", vector_layer.delete_dop))
    def delete(self, location_name, mapset_name, vector_name):
        """Delete an existing vector map layer."""
        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=vector_name,
        )

        if rdc:
            enqueue_job(
                self.job_timeout,
                start_delete_job,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish(0.1)
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("post", vector_layer.post_doc))
    def post(self, location_name, mapset_name, vector_name):
        """Create a new vector layer by uploading a GPKG, zipped Shapefile,
        or GeoJSON.
        """

        allowed_extensions = ["gpkg", "zip", "json", "geojson"]

        # TODO check if another content type can be used
        content_type = request.content_type.split(";")[0]
        if content_type != "multipart/form-data":
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="Content type is not 'multipart/form-data'",
                    )
                ),
                400,
            )

        if "file" not in request.files:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="No file part indicated in postbody.",
                    )
                ),
                400,
            )

        # create download cache path if it does not exist
        if os.path.exists(self.download_cache):
            pass
        else:
            os.mkdir(self.download_cache)

        # save file from request
        id = str(uuid4())
        file = request.files["file"]
        if file.filename == "":
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="No selected file"
                    )
                ),
                400,
            )

        if allowed_file(file.filename, allowed_extensions):
            name, extension = secure_filename(file.filename).rsplit(".", 1)
            filename = f"{name}_{id}.{extension}"
            file_path = os.path.join(self.download_cache, filename)
            file.save(file_path)
        else:
            os.remove(file_path)
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="File has a not allowed extension. "
                        f"Please use {','.join(allowed_extensions)}.",
                    )
                ),
                400,
            )

        # Shapefile upload as zip
        if extension == "zip":
            unzip_folder = os.path.join(self.download_cache, f"unzip_{id}")
            with ZipFile(file_path, "r") as zip_ref:
                zip_ref.extractall(unzip_folder)
            shp_files = [
                entry
                for entry in os.listdir(unzip_folder)
                if entry.endswith(".shp")
            ]
            if len(shp_files) == 0:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="No .shp file found in zip file.",
                        )
                    ),
                    400,
                )
            elif len(shp_files) > 1:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message=f"{len(shp_files)} .shp files found in zip"
                            " file. Please put only one in the zip file.",
                        )
                    ),
                    400,
                )
            else:
                os.remove(file_path)
                file_path = os.path.join(unzip_folder, shp_files[0])

        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=vector_name,
        )
        if rdc:
            rdc.set_request_data(file_path)
            enqueue_job(self.job_timeout, start_create_job, rdc)

        http_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), http_code)
